@@ -9,23 +9,24 @@
   including commercial applications, and to alter it and redistribute it
   freely.
 */
-#include <stdlib.h>
 
-#ifdef __EMSCRIPTEN__
+#include <SDL3/SDL_test_common.h>
+#include <SDL3/SDL_main.h>
+#include "testutils.h"
+
+#ifdef SDL_PLATFORM_EMSCRIPTEN
 #include <emscripten/emscripten.h>
 #endif
 
-#include "SDL_test_common.h"
-#include "testutils.h"
+#include <stdlib.h>
 
-#if defined(__IPHONEOS__) || defined(__ANDROID__) || defined(__EMSCRIPTEN__) || defined(__NACL__) \
-    || defined(__WINDOWS__) || defined(__LINUX__)
+#if defined(SDL_PLATFORM_IOS) || defined(SDL_PLATFORM_ANDROID) || defined(SDL_PLATFORM_EMSCRIPTEN) || defined(SDL_PLATFORM_WIN32) || defined(SDL_PLATFORM_LINUX)
 #define HAVE_OPENGLES2
 #endif
 
 #ifdef HAVE_OPENGLES2
 
-#include "SDL_opengles2.h"
+#include <SDL3/SDL_opengles2.h>
 
 typedef struct GLES2_Context
 {
@@ -35,9 +36,9 @@ typedef struct GLES2_Context
 } GLES2_Context;
 
 static SDL_Surface *g_surf_sdf = NULL;
-GLenum g_texture;
-GLenum g_texture_type = GL_TEXTURE_2D;
-GLfloat g_verts[24];
+static GLenum g_texture;
+static GLenum g_texture_type = GL_TEXTURE_2D;
+static GLfloat g_verts[24];
 typedef enum
 {
     GLES2_ATTRIBUTE_POSITION = 0,
@@ -53,22 +54,19 @@ typedef enum
     GLES2_UNIFORM_COLOR,
 } GLES2_Uniform;
 
-
-GLint g_uniform_locations[16];
+static GLint g_uniform_locations[16];
 
 static SDLTest_CommonState *state;
 static SDL_GLContext *context = NULL;
 static int depth = 16;
 static GLES2_Context ctx;
 
-static int LoadContext(GLES2_Context *data)
+static bool LoadContext(GLES2_Context *data)
 {
 #ifdef SDL_VIDEO_DRIVER_UIKIT
 #define __SDL_NOGETPROCADDR__
 #elif defined(SDL_VIDEO_DRIVER_ANDROID)
 #define __SDL_NOGETPROCADDR__
-#elif defined(SDL_VIDEO_DRIVER_PANDORA)
-#define __SDL_NOGETPROCADDR__
 #endif
 
 #if defined __SDL_NOGETPROCADDR__
@@ -76,7 +74,7 @@ static int LoadContext(GLES2_Context *data)
 #else
 #define SDL_PROC(ret, func, params)                                                            \
     do {                                                                                       \
-        data->func = SDL_GL_GetProcAddress(#func);                                             \
+        data->func = (ret (APIENTRY *) params)SDL_GL_GetProcAddress(#func);                    \
         if (!data->func) {                                                                     \
             return SDL_SetError("Couldn't load GLES2 function %s: %s", #func, SDL_GetError()); \
         }                                                                                      \
@@ -85,7 +83,7 @@ static int LoadContext(GLES2_Context *data)
 
 #include "../src/render/opengles2/SDL_gles2funcs.h"
 #undef SDL_PROC
-    return 0;
+    return true;
 }
 
 /* Call this instead of exit(), so we can clean up SDL: atexit() is evil. */
@@ -97,7 +95,7 @@ quit(int rc)
     if (context) {
         for (i = 0; i < state->num_windows; i++) {
             if (context[i]) {
-                SDL_GL_DeleteContext(context[i]);
+                SDL_GL_DestroyContext(context[i]);
             }
         }
 
@@ -105,7 +103,10 @@ quit(int rc)
     }
 
     SDLTest_CommonQuit(state);
-    exit(rc);
+    /* Let 'main()' return normally */
+    if (rc != 0) {
+        exit(rc);
+    }
 }
 
 #define GL_CHECK(x)                                                                         \
@@ -118,15 +119,14 @@ quit(int rc)
         }                                                                                   \
     }
 
-/*
+/**
  * Create shader, load in source, compile, dump debug as necessary.
  *
  * shader: Pointer to return created shader ID.
  * source: Passed-in shader source code.
  * shader_type: Passed to GL, e.g. GL_VERTEX_SHADER.
  */
-void
-process_shader(GLenum *shader, const char *source, GLenum shader_type)
+static void process_shader(GLenum *shader, const char *source, GLenum shader_type)
 {
     GLint status = GL_FALSE;
     const char *shaders[1] = { NULL };
@@ -253,7 +253,7 @@ typedef struct shader_data
 } shader_data;
 
 static void
-Render(int width, int height, shader_data* data)
+Render(int width, int height, shader_data *data)
 {
     float *verts = g_verts;
     ctx.glViewport(0, 0, 640, 480);
@@ -270,11 +270,11 @@ Render(int width, int height, shader_data* data)
     GL_CHECK(ctx.glDrawArrays(GL_TRIANGLE_STRIP, 0, 4));
 }
 
-void renderCopy_angle(float degree_angle)
+static void renderCopy_angle(float degree_angle)
 {
     const float radian_angle = (float)(3.141592 * degree_angle) / 180.0f;
-    const GLfloat s = (GLfloat) SDL_sin(radian_angle);
-    const GLfloat c = (GLfloat) SDL_cos(radian_angle) - 1.0f;
+    const GLfloat s = (GLfloat)SDL_sin(radian_angle);
+    const GLfloat c = (GLfloat)SDL_cos(radian_angle) - 1.0f;
     GLfloat *verts = g_verts + 16;
     *(verts++) = s;
     *(verts++) = c;
@@ -286,7 +286,7 @@ void renderCopy_angle(float degree_angle)
     *(verts++) = c;
 }
 
-void renderCopy_position(SDL_Rect *srcrect, SDL_Rect *dstrect)
+static void renderCopy_position(SDL_Rect *srcrect, SDL_Rect *dstrect)
 {
     GLfloat minx, miny, maxx, maxy;
     GLfloat minu, maxu, minv, maxv;
@@ -297,10 +297,10 @@ void renderCopy_position(SDL_Rect *srcrect, SDL_Rect *dstrect)
     maxx = (GLfloat)(dstrect->x + dstrect->w);
     maxy = (GLfloat)(dstrect->y + dstrect->h);
 
-    minu = (GLfloat) srcrect->x / (GLfloat)g_surf_sdf->w;
-    maxu = (GLfloat) (srcrect->x + srcrect->w) / (GLfloat)g_surf_sdf->w;
-    minv = (GLfloat) srcrect->y / (GLfloat)g_surf_sdf->h;
-    maxv = (GLfloat) (srcrect->y + srcrect->h) / (GLfloat)g_surf_sdf->h;
+    minu = (GLfloat)srcrect->x / (GLfloat)g_surf_sdf->w;
+    maxu = (GLfloat)(srcrect->x + srcrect->w) / (GLfloat)g_surf_sdf->w;
+    minv = (GLfloat)srcrect->y / (GLfloat)g_surf_sdf->h;
+    maxv = (GLfloat)(srcrect->y + srcrect->h) / (GLfloat)g_surf_sdf->h;
 
     *(verts++) = minx;
     *(verts++) = miny;
@@ -321,23 +321,22 @@ void renderCopy_position(SDL_Rect *srcrect, SDL_Rect *dstrect)
     *(verts++) = maxv;
 }
 
-int done;
-Uint32 frames;
-shader_data *datas;
+static int done;
+static Uint32 frames;
+static shader_data *datas;
 
-void loop(void)
+static void loop(void)
 {
     SDL_Event event;
     int i;
-    int status;
 
     /* Check for events */
     ++frames;
     while (SDL_PollEvent(&event) && !done) {
         switch (event.type) {
-        case SDL_KEYDOWN:
+        case SDL_EVENT_KEY_DOWN:
         {
-            const int sym = event.key.keysym.sym;
+            const int sym = event.key.key;
 
             if (sym == SDLK_TAB) {
                 SDL_Log("Tab");
@@ -359,30 +358,29 @@ void loop(void)
             break;
         }
 
-        case SDL_WINDOWEVENT:
-            switch (event.window.event) {
-            case SDL_WINDOWEVENT_RESIZED:
-                for (i = 0; i < state->num_windows; ++i) {
-                    if (event.window.windowID == SDL_GetWindowID(state->windows[i])) {
-                        int w, h;
-                        status = SDL_GL_MakeCurrent(state->windows[i], context[i]);
-                        if (status) {
-                            SDL_Log("SDL_GL_MakeCurrent(): %s\n", SDL_GetError());
-                            break;
-                        }
-                        /* Change view port to the new window dimensions */
-                        SDL_GL_GetDrawableSize(state->windows[i], &w, &h);
-                        ctx.glViewport(0, 0, w, h);
-                        state->window_w = event.window.data1;
-                        state->window_h = event.window.data2;
-                        /* Update window content */
-                        Render(event.window.data1, event.window.data2, &datas[i]);
-                        SDL_GL_SwapWindow(state->windows[i]);
+        case SDL_EVENT_WINDOW_RESIZED:
+            for (i = 0; i < state->num_windows; ++i) {
+                if (event.window.windowID == SDL_GetWindowID(state->windows[i])) {
+                    int w, h;
+                    if (!SDL_GL_MakeCurrent(state->windows[i], context[i])) {
+                        SDL_Log("SDL_GL_MakeCurrent(): %s\n", SDL_GetError());
                         break;
                     }
+                    /* Change view port to the new window dimensions */
+                    SDL_GetWindowSizeInPixels(state->windows[i], &w, &h);
+                    ctx.glViewport(0, 0, w, h);
+                    state->window_w = event.window.data1;
+                    state->window_h = event.window.data2;
+                    /* Update window content */
+                    Render(event.window.data1, event.window.data2, &datas[i]);
+                    SDL_GL_SwapWindow(state->windows[i]);
+                    break;
                 }
-                break;
             }
+            break;
+
+        default:
+            break;
         }
         SDLTest_CommonEvent(state, &event, &done);
     }
@@ -400,18 +398,22 @@ void loop(void)
         int w, h;
         SDL_Rect rs, rd;
 
-        SDL_GL_GetDrawableSize(state->windows[0], &w, &h);
+        SDL_GetWindowSizeInPixels(state->windows[0], &w, &h);
 
-        rs.x = 0; rs.y = 0; rs.w = g_surf_sdf->w; rs.h = g_surf_sdf->h;
-        rd.w = (int)((float)g_surf_sdf->w * g_val); rd.h = (int)((float)g_surf_sdf->h * g_val);
-        rd.x = (w - rd.w) / 2; rd.y = (h - rd.h) / 2;
+        rs.x = 0;
+        rs.y = 0;
+        rs.w = g_surf_sdf->w;
+        rs.h = g_surf_sdf->h;
+        rd.w = (int)((float)g_surf_sdf->w * g_val);
+        rd.h = (int)((float)g_surf_sdf->h * g_val);
+        rd.x = (w - rd.w) / 2;
+        rd.y = (h - rd.h) / 2;
         renderCopy_position(&rs, &rd);
     }
 
     if (!done) {
         for (i = 0; i < state->num_windows; ++i) {
-            status = SDL_GL_MakeCurrent(state->windows[i], context[i]);
-            if (status) {
+            if (!SDL_GL_MakeCurrent(state->windows[i], context[i])) {
                 SDL_Log("SDL_GL_MakeCurrent(): %s\n", SDL_GetError());
 
                 /* Continue for next window */
@@ -421,7 +423,7 @@ void loop(void)
             SDL_GL_SwapWindow(state->windows[i]);
         }
     }
-#ifdef __EMSCRIPTEN__
+#ifdef SDL_PLATFORM_EMSCRIPTEN
     else {
         emscripten_cancel_main_loop();
     }
@@ -433,9 +435,8 @@ int main(int argc, char *argv[])
     int fsaa, accel;
     int value;
     int i;
-    SDL_DisplayMode mode;
-    Uint32 then, now;
-    int status;
+    const SDL_DisplayMode *mode;
+    Uint64 then, now;
     shader_data *data;
     char *path = NULL;
 
@@ -464,8 +465,13 @@ int main(int argc, char *argv[])
                 if (!argv[i]) {
                     consumed = -1;
                 } else {
-                    depth = SDL_atoi(argv[i]);
-                    consumed = 1;
+                    char *endptr = NULL;
+                    depth = (int)SDL_strtol(argv[i], &endptr, 0);
+                    if (endptr != argv[i] && *endptr == '\0') {
+                        consumed = 1;
+                    } else {
+                        consumed = -1;
+                    }
                 }
             } else {
                 consumed = -1;
@@ -517,7 +523,7 @@ int main(int argc, char *argv[])
     }
 
     /* Important: call this *after* creating the context */
-    if (LoadContext(&ctx) < 0) {
+    if (!LoadContext(&ctx)) {
         SDL_Log("Could not load GLES2 functions\n");
         quit(2);
         return 0;
@@ -590,64 +596,56 @@ int main(int argc, char *argv[])
         TTF_CloseFont(font);
         TTF_Quit();
 #endif
-        g_surf_sdf = SDL_ConvertSurfaceFormat(tmp, SDL_PIXELFORMAT_ABGR8888, 0);
+        g_surf_sdf = SDL_ConvertSurface(tmp, SDL_PIXELFORMAT_ABGR8888);
 
         SDL_SetSurfaceBlendMode(g_surf_sdf, SDL_BLENDMODE_BLEND);
     }
 
-    if (state->render_flags & SDL_RENDERER_PRESENTVSYNC) {
-        SDL_GL_SetSwapInterval(1);
-    } else {
-        SDL_GL_SetSwapInterval(0);
-    }
+    SDL_GL_SetSwapInterval(state->render_vsync);
 
-    SDL_GetCurrentDisplayMode(0, &mode);
-    SDL_Log("Screen bpp: %d\n", SDL_BITSPERPIXEL(mode.format));
-    SDL_Log("\n");
+    mode = SDL_GetCurrentDisplayMode(SDL_GetPrimaryDisplay());
+    if (mode) {
+        SDL_Log("Screen bpp: %d\n", SDL_BITSPERPIXEL(mode->format));
+        SDL_Log("\n");
+    }
     SDL_Log("Vendor     : %s\n", ctx.glGetString(GL_VENDOR));
     SDL_Log("Renderer   : %s\n", ctx.glGetString(GL_RENDERER));
     SDL_Log("Version    : %s\n", ctx.glGetString(GL_VERSION));
     SDL_Log("Extensions : %s\n", ctx.glGetString(GL_EXTENSIONS));
     SDL_Log("\n");
 
-    status = SDL_GL_GetAttribute(SDL_GL_RED_SIZE, &value);
-    if (!status) {
+    if (SDL_GL_GetAttribute(SDL_GL_RED_SIZE, &value)) {
         SDL_Log("SDL_GL_RED_SIZE: requested %d, got %d\n", 5, value);
     } else {
         SDL_Log("Failed to get SDL_GL_RED_SIZE: %s\n",
                 SDL_GetError());
     }
-    status = SDL_GL_GetAttribute(SDL_GL_GREEN_SIZE, &value);
-    if (!status) {
+    if (SDL_GL_GetAttribute(SDL_GL_GREEN_SIZE, &value)) {
         SDL_Log("SDL_GL_GREEN_SIZE: requested %d, got %d\n", 5, value);
     } else {
         SDL_Log("Failed to get SDL_GL_GREEN_SIZE: %s\n",
                 SDL_GetError());
     }
-    status = SDL_GL_GetAttribute(SDL_GL_BLUE_SIZE, &value);
-    if (!status) {
+    if (SDL_GL_GetAttribute(SDL_GL_BLUE_SIZE, &value)) {
         SDL_Log("SDL_GL_BLUE_SIZE: requested %d, got %d\n", 5, value);
     } else {
         SDL_Log("Failed to get SDL_GL_BLUE_SIZE: %s\n",
                 SDL_GetError());
     }
-    status = SDL_GL_GetAttribute(SDL_GL_DEPTH_SIZE, &value);
-    if (!status) {
+    if (SDL_GL_GetAttribute(SDL_GL_DEPTH_SIZE, &value)) {
         SDL_Log("SDL_GL_DEPTH_SIZE: requested %d, got %d\n", depth, value);
     } else {
         SDL_Log("Failed to get SDL_GL_DEPTH_SIZE: %s\n",
                 SDL_GetError());
     }
     if (fsaa) {
-        status = SDL_GL_GetAttribute(SDL_GL_MULTISAMPLEBUFFERS, &value);
-        if (!status) {
+        if (SDL_GL_GetAttribute(SDL_GL_MULTISAMPLEBUFFERS, &value)) {
             SDL_Log("SDL_GL_MULTISAMPLEBUFFERS: requested 1, got %d\n", value);
         } else {
             SDL_Log("Failed to get SDL_GL_MULTISAMPLEBUFFERS: %s\n",
                     SDL_GetError());
         }
-        status = SDL_GL_GetAttribute(SDL_GL_MULTISAMPLESAMPLES, &value);
-        if (!status) {
+        if (SDL_GL_GetAttribute(SDL_GL_MULTISAMPLESAMPLES, &value)) {
             SDL_Log("SDL_GL_MULTISAMPLESAMPLES: requested %d, got %d\n", fsaa,
                     value);
         } else {
@@ -656,8 +654,7 @@ int main(int argc, char *argv[])
         }
     }
     if (accel) {
-        status = SDL_GL_GetAttribute(SDL_GL_ACCELERATED_VISUAL, &value);
-        if (!status) {
+        if (SDL_GL_GetAttribute(SDL_GL_ACCELERATED_VISUAL, &value)) {
             SDL_Log("SDL_GL_ACCELERATED_VISUAL: requested 1, got %d\n", value);
         } else {
             SDL_Log("Failed to get SDL_GL_ACCELERATED_VISUAL: %s\n",
@@ -671,8 +668,7 @@ int main(int argc, char *argv[])
     for (i = 0; i < state->num_windows; ++i) {
 
         int w, h;
-        status = SDL_GL_MakeCurrent(state->windows[i], context[i]);
-        if (status) {
+        if (!SDL_GL_MakeCurrent(state->windows[i], context[i])) {
             SDL_Log("SDL_GL_MakeCurrent(): %s\n", SDL_GetError());
 
             /* Continue for next window */
@@ -699,7 +695,7 @@ int main(int argc, char *argv[])
             GL_CHECK(ctx.glTexSubImage2D(g_texture_type, 0, 0 /* xoffset */, 0 /* yoffset */, g_surf_sdf->w, g_surf_sdf->h, format, type, g_surf_sdf->pixels));
         }
 
-        SDL_GL_GetDrawableSize(state->windows[i], &w, &h);
+        SDL_GetWindowSizeInPixels(state->windows[i], &w, &h);
         ctx.glViewport(0, 0, w, h);
 
         data = &datas[i];
@@ -747,7 +743,7 @@ int main(int argc, char *argv[])
         ctx.glBindTexture(g_texture_type, g_texture);
         GL_CHECK(ctx.glClearColor(1, 1, 1, 1));
 
-        // SDL_BLENDMODE_BLEND
+        /* SDL_BLENDMODE_BLEND */
         GL_CHECK(ctx.glEnable(GL_BLEND));
         ctx.glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
         ctx.glBlendEquationSeparate(GL_FUNC_ADD, GL_FUNC_ADD);
@@ -758,7 +754,7 @@ int main(int argc, char *argv[])
     then = SDL_GetTicks();
     done = 0;
 
-#ifdef __EMSCRIPTEN__
+#ifdef SDL_PLATFORM_EMSCRIPTEN
     emscripten_set_main_loop(loop, 0, 1);
 #else
     while (!done) {
@@ -772,7 +768,7 @@ int main(int argc, char *argv[])
         SDL_Log("%2.2f frames per second\n",
                 ((double)frames * 1000) / (now - then));
     }
-#if !defined(__ANDROID__) && !defined(__NACL__)
+#ifndef SDL_PLATFORM_ANDROID
     quit(0);
 #endif
     return 0;
@@ -787,5 +783,3 @@ int main(int argc, char *argv[])
 }
 
 #endif /* HAVE_OPENGLES2 */
-
-/* vi: set ts=4 sw=4 expandtab: */
